
// TEMPERAGE DU CHOCOLAT
// Programme conçu en mai 2019 par Anne-Laure ALLEGRE et Marie-Anne DEJOAN dans le cadre du GRAC "Nouveaux programmes"
// Académie de la Guyane

// Montage pont diviseur avec thermistance + buzzer et 2 interrupteurs à glissière
// fonctions de base arduino: http://www.mon-club-elec.fr/pmwiki_reference_arduino/pmwiki.php?n=Main.Reference

// L'objectif de ce programme est d'être averti par des bips sonores (1, 2 ou 3) des températures "clés" atteintes par le chocolat pour suivre au mieux les trois phases du tempérage (Phase 1, 2 et 3)
// La thermistance du montage pont diviseur est plongée dans le chocolat que l'on souhaite tempérer
// Phase 1: Fonte du chocolat au bain marie, le buzzer bippe 1 fois lorsqu'on atteint la température de fonte du chocolat
// Phase 2: Refroidissement du chocolat en plongeant le contenant dans de l'eau froide, le buzzer bippe 2 fois lorsqu'on atteint la température de refroidissement du chocolat
// Phase 3: Rechauffage du chocolat pour obtenir du chocolat tempéré, le buzzer bippe 3 fois lorsqu'on atteint la température du chocolat tempéré
// Le chocolat tempéré peut être utilisé pour réaliser des bonbons, oeufs de Pâques, etc, brillants et appétissants




// Définition des constantes : températures "clés" pour le tempérage du chocolat
// REMARQUE: on utilise la commande "const float" pour les nombres à virgules (float signifie "flottant" en français)

const float TempChocFondu = 34.0; // Température que l'on souhaite atteintre pour faire fondre le chocolat - Pour le chocolat noir 45°/50°C
const float TempChocRefroidi = 29.0; // Température que l'on souhaite atteintre lorsqu'on refroidit le chocolat - Pour le chocolat noir 26°/27°C
const float TempChocTempere = 31.0; // Température que l'on souhaite atteintre lorsqu'on l'on fait rechauffer le chocolat pour obtenir du chocolat tempéré - Pour le chocolat noir 31/32°C


// Définition des broches numériques utilisées du microcontrôleur
// REMARQUE: on utilise la commande "const int" pour les nombres entiers (int: integer, signifie "entier" en français)

const int Buzzer = 10; // Broche 10 utilisée pour le buzzer piezoélectrique - Elle se nomme maintenant : "Buzzer"
const int IG1 = 8; // Broche 8 utilisée pour l'interrupteur à glissière n°1 - Elle se nomme maintenant : "IG1"
const int IG2 = 9; // Broche 9 utilisée pour l'interrupteur à glissière n°2 - Elle se nomme maintenant : "IG2"


// Définition des entrées analogiques utilisées du microcontrôleur

const int Vout = A0; // Entrée analogique A0 utilisée pour la tension de sortie Vout du pont diviseur - Elle se nomme maintenant : "Vout"




// BOUCLE D'INITIALISATION : exécutée une seule fois
// Définition des broches numériques d'entrée et de sortie du microcontrôleur



void setup()
{
  Serial.begin(9600); // ouvre le port série et fixe le debit de communication à 9600 bauds (nombre de symboles transmissibles par seconde)
  pinMode(Buzzer, OUTPUT); // "Buzzer" est une broche de sortie
  pinMode(IG1, INPUT); // "IG1" est une broche d'entrée
  pinMode(IG2, INPUT); // "IG2" est une broche d'entrée
}




// BOUCLE PRINCIPALE : exécutée à l'infini

void loop()
{

  // Affichage de la tension Vout en sortie du montage pont diviseur sur le Moniteur série

  // REMARQUE : Nous n'avons pas accès directement à la tension Vout. En effet, les broches analogiques de la carte Arduino sont reliées à un Convertisseur Analogique Numérique (CAN) 10 bits.
  // Ainsi, la tension Vout, comprise entre 0 et 5V a été transformée en une valeur numérique entière comprise entre 0 et 1023.
  // POur afficher la tension Vout, il faut donc convertir cette valeur numérique(à laquelle on a accès) en tension.

  int VoutVal = analogRead(Vout); // Lecture de la valeur numérique (comprise en 0 et 1023) en sortie du CAN relié à la broche analogique "Vout"et stockage du résultat dans "VoutVal"
  float TensionVout = (VoutVal / 1024.0) * 5.0; // Conversion de la valeur numérique en tension : la valeur numérique comprise entre 0 et 1023 correspond à une tension comprise entre 0 et 5V
  Serial.print("Tension Vout : "); //Affichage dans le Moniteur Série de "Tension Vout (V): "
  Serial.print(TensionVout);// Affichage dans le Moniteur Série de la valeur de la tension Vout
  Serial.print("V");//Affichage dans le Moniteur Série de "V"

  // Affichage de la température du chocolat

  // REMARQUE: La température du chocolat téta est obtenue à partir de la tension Vout en utilisant l'équation de la courbe d'étalonnage: Vout=f(téta)

  float teta = (TensionVout - 1.3) / 0.034; // Relation déterminée expérimentalement (modélisation) grace à la courbe d'étalonnage préalablement réalisée : Uout = 0.0426*Théta + 1.38 (Remarque : j'ai un peu triché et adapté l'équation de telle sorte que le moniteur série affiche la bonne température!!)
  Serial.print("                  Température chocolat : ");//Affichage dans le Moniteur Série de ", Température chocolat : "
  Serial.print(teta); // Affichage dans le Moniteur Série de la valeur de la température du chocolat téta
  Serial.println("°C");//Affichage dans le Moniteur Série de "°C" - La commande Serial.println effectue un saut de ligne dans le moniteur après avoir envoyé la valeur de la variable, rend l'affichage plus clair


  // Lecture des entrées numériques

  int InterGlissiere1 = digitalRead(IG1); // Lecture de l'entree numérique IG1 et stockage du résultats dans InterGlissiere1
  int InterGlissiere2 = digitalRead(IG2); // Lecture de l'entree numérique IG2 et stockage du résultats dans InterGlissiere2


  //Phase 1: Fonte du chocolat, les interrupteurs à glissière 1 et 2 sont ouverts, le buzzer bippe 1 fois lorsqu'on atteint la température de fonte du chocolat - Pour le chocolat noir 45°/50°C

  if (InterGlissiere1 == LOW && InterGlissiere2 == LOW) // Si "InterGlissiere1" et "InterGlissiere2" sont à l'état BAS (les interrupteurs sont ouverts)
  {
    if (teta >= TempChocFondu) // Si la température de fonte du chocolat est atteinte
    {
      digitalWrite(Buzzer, HIGH); // Met un niveau logique HAUT sur la broche numérique "Buzzer"; La tension de la broche est mise à 5V - Le buzzer est alimenté
      tone (10, 100, 200); // Le buzzer bipe à 100 Hz pendant 200ms
      Serial.print("IL FAUT REFROIDIR LE CHOCOLAT                  "); // Affichage dans le Moniteur Série de "IL FAUT REFROIDIR LE CHOCOLAT "
      delay(700); // Pause pendant 700 ms entre les bips
    }
    else   // Sinon
    {
      digitalWrite(Buzzer, LOW); //Met un niveau logique BAS sur la broche numérique "Buzzer"; La tension de la broche est mise à 0V - Le buzzer n'est pas alimenté
      delay(10); // Pause pendant 10 ms
    }
  }

  //Phase 2: Refroidissement du chocolat, l'interrupteur à glissière 1 est fermé et le 2 est ouvert, le buzzer bippe 2 fois lorsqu'on atteint la température de refroidissement du chocolat - Pour le chocolat noir 26°/27°C

  else if (InterGlissiere1 == HIGH && InterGlissiere2 == LOW) // Sinon si "InterGlissiere1" est à l'état HAUT et "InterGlissiere2" à l'état BAS (L'interrupteur 1 est fermé et le 2 est ouvert)
  {
    if (teta <= TempChocRefroidi) // Et si la température de refroidissement du chocolat est atteinte
    {
      for (int i = 0; i < 2; i++) // Boucle qui commande le nombre de bips à executer (i allant de 0 à 1)- Ici 2 bips
      {
        digitalWrite(Buzzer, HIGH); // Met un niveau logique HAUT sur la broche numérique "Buzzer"; La tension de la broche est mise à 5V - Le buzzer est alimenté
        tone (10, 100, 200); // Le buzzer bipe à 100 Hz pendant 200ms
        delay(400); // Pause de 400 ms entre chaque bip
      }
      Serial.print("IL FAUT RECHAUFFER LE CHOCOLAT                  "); // Affichage dans le Moniteur Série de "IL FAUT RECHAUFFER LE CHOCOLAT "
      delay(300); // Pause pendant 300 ms après les 2 bips
    }
    else // Sinon
    {
      digitalWrite(Buzzer, LOW);//Met un niveau logique BAS sur la broche numérique "Buzzer"; La tension de la broche est mise à 0V - Le buzzer n'est pas alimenté
      delay(100); // Pause pendant 10 ms
    }
  }


  //Phase 3: Rechauffage du chocolat pour obtenir du chocolat tempéré, les interrupteurs à glissière 1 et 2 sont fermés, le buzzer bippe 3 fois lorsqu'on atteint la température du chocolat tempéré - Pour le chocolat noir 31°/32°C

  else if (InterGlissiere1 == HIGH && InterGlissiere2 == HIGH) // Sinon si InterGlissiere1 et InterGlissiere2 sont à l'état HAUT (les interrupteurs sont fermés)
  {
    if (teta >= TempChocTempere && teta <= TempChocTempere + 1) // Et si la température du chocolat tempéré est atteinte
    {
      for (int i = 0; i < 3; i++) // Boucle qui commande le nombre de bips à executer (i allant de 0 à 2)- Ici 3 bips
      {
        digitalWrite(Buzzer, HIGH); // Met un niveau logique HAUT sur la broche numérique "Buzzer"; La tension de la broche est mise à 5V - Le buzzer est alimenté
        tone (10, 100, 200); // Le buzzer bipe à 100 Hz pendant 200ms
        delay(400); // Pause de 400 ms entre chaque bip
      }
      Serial.print("LE CHOCOLAT EST TEMPERE, BRAVO!!!                  "); // Affichage dans le Moniteur Série de "LE CHOCOLAT EST TEMPERE, BRAVO!!! "
      delay(300); // Pause pendant 300 ms après les 3 bips
    }
    else // Sinon
    {
      digitalWrite(Buzzer, LOW);//Met un niveau logique BAS sur la broche numérique "Buzzer"; La tension de la broche est mise à 0V - Le buzzer n'est pas alimenté
      delay(10); // Pause pendant 10 ms
    }
  }
  else // Instruction qui vient complétée if() et qui permet qu'autre chose se passe si la 1ère condition n'est pas remplie (sinon)
  {
    digitalWrite(Buzzer, LOW);//Met un niveau logique BAS sur la broche numérique "Buzzer"; La tension de la broche est mise à 0V - Le buzzer n'est pas alimenté
    delay(10); // Pause pendant 10 ms
  }

}

// Le chocolat tempéré peut être utilisé pour réaliser des bonbons, oeufs de Pâques, etc, brillants et appétissants!
